"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const items_1 = __importDefault(require("./items"));
const user_defined_function_1 = __importDefault(require("./user-defined-function"));
const resource_id_1 = __importDefault(require("./resource-id"));
function define(body) {
    // eslint-disable-next-line no-new-func
    return new Function(`"use strict";return (${body})`)();
}
class UserDefinedFunctions extends items_1.default {
    constructor(parent) {
        super(parent, ["/id"]);
        this.functions = {};
    }
    _newItem(data) {
        return new user_defined_function_1.default(data);
    }
    _rid(id) {
        const idBuffer = Buffer.alloc(8, 0);
        idBuffer.writeInt32LE(id, 0);
        idBuffer.writeInt8(resource_id_1.default.UserDefinedFunctionByte << 4, 7);
        const idString = resource_id_1.default.bigNumberReadIntBE(idBuffer, 0, 8).toString();
        const collection = this._parent.read();
        const rid = resource_id_1.default.parse(collection._rid);
        rid.userDefinedFunction = idString;
        return rid.toString();
    }
    _self(rid) {
        const parent = this._parent.read();
        if (!parent) {
            throw new Error("parent is missing");
        }
        return `${parent._self}/udfs/${rid}`;
    }
    replace({ id, body }) {
        const oldData = this._item("/id", id, id).read();
        if (!oldData) {
            throw new Error("does not exist");
        }
        this.functions[oldData.id] = define(body);
        return super.replace({ id, body }, oldData);
    }
    create({ id, body }) {
        this.functions[id] = define(body);
        return super.create({ id, body });
    }
    upsert({ id, body }) {
        this.functions[id] = define(body);
        return super.upsert({ id, body });
    }
    delete(idOrRid) {
        const data = this._item("/id", idOrRid, idOrRid).read();
        if (!data) {
            throw new Error("does not exist");
        }
        delete this.functions[data.id];
        return super.delete(idOrRid, idOrRid);
    }
}
exports.default = UserDefinedFunctions;
