"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cosmosdb_query_1 = require("@zeit/cosmosdb-query");
const parseJSONOrNull = (s) => {
    try {
        return JSON.parse(s);
    }
    catch (err) {
        return null;
    }
};
const parseHeaders = (headers) => {
    const maxItemCount = headers["x-ms-max-item-count"]
        ? parseInt(headers["x-ms-max-item-count"], 10)
        : null;
    const continuation = headers["x-ms-continuation"]
        ? parseJSONOrNull(headers["x-ms-continuation"])
        : null;
    return { maxItemCount, continuation };
};
exports.default = async (req, res, itemsName, fn) => {
    const { maxItemCount, continuation } = parseHeaders(req.headers);
    let result;
    try {
        result = await fn({ maxItemCount, continuation });
    }
    catch (err) {
        if (err instanceof cosmosdb_query_1.SyntaxError) {
            res.statusCode = 400;
            return {
                code: "BadRequest",
                message: err.message
            };
        }
        throw err;
    }
    if (!result) {
        res.statusCode = 404;
        return {};
    }
    if (!result.result) {
        return result;
    }
    const count = result.result.length;
    if (result.continuation) {
        res.setHeader("x-ms-continuation", JSON.stringify(result.continuation));
    }
    res.setHeader("x-ms-item-count", String(count));
    return { [itemsName]: result.result, _count: count };
};
