"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const documentOperations = __importStar(require("./_document-operations"));
const json_1 = __importDefault(require("../json"));
const true_header_1 = __importDefault(require("../true-header"));
const types_1 = require("../types");
exports.default = async (account, req, res, { dbId, collId }) => {
    const body = await (0, json_1.default)(req);
    const isAtomic = (0, true_header_1.default)(req, "x-ms-cosmos-batch-atomic");
    const continueOnError = (0, true_header_1.default)(req, "x-ms-cosmos-batch-continue-on-error");
    const collection = account.database(dbId).collection(collId);
    if (!collection.read()) {
        res.statusCode = 404;
        return {};
    }
    if (!Array.isArray(body)) {
        res.statusCode = 400;
        return { message: "Invalid request body" };
    }
    const responseBody = [];
    let failed = false;
    for (let i = 0, l = body.length; i < l; i += 1) {
        const input = body[i];
        if (failed && !continueOnError) {
            responseBody.push({ statusCode: 424, requestCharge: 0 });
            // eslint-disable-next-line no-continue
            continue;
        }
        const operationType = input ? input.operationType : null;
        let result;
        switch (operationType) {
            case types_1.BulkOperationType.Create:
                result = documentOperations.createOperation(collection, input);
                break;
            case types_1.BulkOperationType.Upsert:
                result = documentOperations.upsertOperation(collection, input);
                break;
            case types_1.BulkOperationType.Read:
                result = documentOperations.readOperation(collection, input);
                break;
            case types_1.BulkOperationType.Delete:
                result = documentOperations.deleteOperation(collection, input);
                break;
            case types_1.BulkOperationType.Replace:
                result = documentOperations.replaceOperation(collection, input);
                break;
            default:
                res.statusCode = 400;
                return {
                    message: `Operation type '${operationType}' is not supported in Batch.`
                };
        }
        failed = result.statusCode >= 400;
        if (failed) {
            responseBody.push({
                statusCode: result.statusCode,
                requestCharge: 0
            });
        }
        else {
            responseBody.push({
                statusCode: result.statusCode,
                requestCharge: 1,
                eTag: result.body ? result.body._etag : undefined,
                resourceBody: result.body
            });
        }
    }
    return responseBody;
};
