"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cosmosdb_query_1 = __importDefault(require("@zeit/cosmosdb-query"));
const _read_items_1 = __importDefault(require("./_read-items"));
const json_1 = __importDefault(require("../json"));
const true_header_1 = __importDefault(require("../true-header"));
exports.default = async (account, req, res, { dbId, collId }) => (0, _read_items_1.default)(req, res, "Documents", async ({ continuation, maxItemCount }) => {
    const body = await (0, json_1.default)(req);
    if (!body.query) {
        res.statusCode = 400;
        return { message: "missing query" };
    }
    const collection = account.database(dbId).collection(collId);
    if (!(0, true_header_1.default)(req, "x-ms-documentdb-query-enablecrosspartition")) {
        const { partitionKey } = collection.read() || {};
        const paths = (partitionKey || {}).paths || [];
        if (paths.length && !(0, cosmosdb_query_1.default)(body.query).containsPartitionKeys(paths)) {
            res.statusCode = 400;
            return { message: "missing partition keys" };
        }
    }
    try {
        return collection.documents.query(body, { continuation, maxItemCount });
    }
    catch (err) {
        if (err.badRequest) {
            res.statusCode = 400;
            return { message: err.message };
        }
        throw err;
    }
});
