"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _document_operations_1 = require("./_document-operations");
const true_header_1 = __importDefault(require("../true-header"));
const get_partition_from_header_1 = __importStar(require("../utils/get-partition-from-header"));
/**
 * Given a collection configuration this filters out _partitionKey which
 * is legacy and parses each partitionKey into a key of the collection
 * data.
 */
function getCollectionPartitionKeys(collection) {
    if (!collection.partitionKey)
        return [];
    return collection.partitionKey.paths
        .map(path => path.slice(1))
        .filter(path => path !== "_partitionKey");
}
/**
 * Given some headers it will parse the provided partition keys into
 * an array of paths. It ignores array of empty object as it is what
 * the client provides when no partition key is given.
 */
function getClientPartitionKeys(headers) {
    const key = headers["x-ms-documentdb-partitionkey"];
    if (!key || key === "[{}]") {
        return [];
    }
    try {
        return JSON.parse(key);
    }
    catch (error) {
        return [];
    }
}
exports.default = (account, req, res, { dbId, collId, docId }) => {
    const collection = account.database(dbId).collection(collId);
    if (!collection.read()) {
        res.statusCode = 404;
        return {
            code: "NotFound",
            message: "Entity with the specified id does not exist in the system.,"
        };
    }
    const partitionKey = (0, get_partition_from_header_1.default)(req);
    if (!(0, true_header_1.default)(req, "x-ms-documentdb-query-enablecrosspartition")) {
        const collectionKeys = getCollectionPartitionKeys(collection.read());
        const clientKeys = getClientPartitionKeys(req.headers);
        if (collectionKeys.length > clientKeys.length) {
            res.statusCode = 400;
            return {
                code: "BadRequest",
                message: "The partition key supplied in x-ms-partitionkey header has fewer components than defined in the collection"
            };
        }
        const parsedPartitionKey = (0, get_partition_from_header_1.parsePartitionKey)(partitionKey, docId);
        const data = collection.document(docId, parsedPartitionKey).read();
        for (let i = 0; i <= collectionKeys.length; i += 1) {
            const path = collectionKeys[i];
            if (data[path] !== clientKeys[i]) {
                res.statusCode = 404;
                return {
                    code: "NotFound",
                    message: "Entity with the specified id does not exist in the system.,"
                };
            }
        }
    }
    const result = (0, _document_operations_1.readOperation)(collection, { partitionKey, id: docId });
    res.statusCode = result.statusCode;
    return result.body;
};
