"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const json_1 = __importDefault(require("../json"));
exports.default = async (account, req, res, { dbId, collId }) => {
    const body = await (0, json_1.default)(req);
    if (!body.id) {
        res.statusCode = 400;
        return { message: "missing id" };
    }
    const database = account.database(dbId);
    const data = database.collection(collId).read();
    if (!data) {
        res.statusCode = 404;
        return {};
    }
    try {
        assert_1.default.deepStrictEqual(body.partitionKey, data.partitionKey);
    }
    catch (err) {
        res.statusCode = 400;
        return { message: "replacing partitionKey is not allowed" };
    }
    try {
        return await database.collections.replace(body, data);
    }
    catch (err) {
        if (err.badRequest) {
            res.statusCode = 400;
            return { message: err.message };
        }
        if (err.conflict) {
            res.statusCode = 409;
            return { message: err.message, code: "Conflict" };
        }
        throw err;
    }
};
