"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHttpsServer = exports.createHttpServer = void 0;
const fs_1 = require("fs");
const http = __importStar(require("http"));
const https = __importStar(require("https"));
const path_1 = require("path");
const crypto_1 = require("crypto");
const account_1 = __importDefault(require("./account"));
const routes_1 = __importDefault(require("./routes"));
const generateRequestHandler = ({ keepAlive = false }) => (account, req, res) => {
    const route = (0, routes_1.default)(req);
    (async () => {
        let body;
        if (route) {
            const [params, handler] = route;
            try {
                body = await handler(account, req, res, params);
            }
            catch (err) {
                // eslint-disable-next-line no-console
                console.error(err);
                body = { message: err.message };
                res.statusCode = 500;
            }
            if (res.statusCode > 399 && !body.message) {
                body.message = "";
            }
        }
        else {
            res.statusCode = 400;
            body = { message: "no route" };
        }
        if (body && body._etag) {
            res.setHeader("etag", body._etag);
        }
        res.setHeader("content-type", "application/json");
        res.setHeader("content-location", `https://${req.headers.host}${req.url}`);
        res.setHeader("connection", keepAlive ? "keep-alive" : "close");
        res.setHeader("x-ms-activity-id", (0, crypto_1.randomUUID)());
        res.setHeader("x-ms-request-charge", "1");
        if (req.headers["x-ms-documentdb-populatequerymetrics"]) {
            res.setHeader("x-ms-documentdb-query-metrics", "totalExecutionTimeInMs=0.00;queryCompileTimeInMs=0.00;queryLogicalPlanBuildTimeInMs=0.00;queryPhysicalPlanBuildTimeInMs=0.00;queryOptimizationTimeInMs=0.00;VMExecutionTimeInMs=0.00;indexLookupTimeInMs=0.00;documentLoadTimeInMs=0.00;systemFunctionExecuteTimeInMs=0.00;userFunctionExecuteTimeInMs=0.00;retrievedDocumentCount=0;retrievedDocumentSize=0;outputDocumentCount=1;outputDocumentSize=0;writeOutputTimeInMs=0.00;indexUtilizationRatio=0.00");
        }
        if (req.headers["a-im"] && res.statusCode === 200) {
            if (req.headers["if-none-match"]) {
                res.statusCode = 304;
            }
            res.setHeader("etag", "1");
        }
        res.end(JSON.stringify(body));
    })().catch(err => {
        // eslint-disable-next-line no-console
        console.error(err);
        if (!res.finished) {
            res.statusCode = 500;
            res.end("");
        }
    });
};
const createAccount = (address) => {
    if (!address || typeof address !== "object") {
        throw new Error(`Unexpected address type: ${address}`);
    }
    const { address: host, port } = address;
    const hostname = host === "0.0.0.0" || host === "::" ? "localhost" : host;
    return new account_1.default(hostname, port);
};
function createHttpServer(opts = {}) {
    let account;
    const handleRequest = generateRequestHandler({
        keepAlive: opts.keepAlive
    });
    const server = http
        .createServer(opts, (req, res) => {
        handleRequest(account, req, res);
    })
        .on("listening", () => {
        account = createAccount(server.address());
    });
    return server;
}
exports.createHttpServer = createHttpServer;
function createHttpsServer(opts) {
    let account;
    const options = {
        cert: (0, fs_1.readFileSync)((0, path_1.join)(__dirname, "..", "cert.pem")),
        key: (0, fs_1.readFileSync)((0, path_1.join)(__dirname, "..", "key.pem")),
        minVersion: "TLSv1",
        rejectUnauthorized: false,
        requestCert: false,
        ...opts
    };
    const handleRequest = generateRequestHandler({
        keepAlive: options.keepAlive
    });
    const server = https
        .createServer(options, (req, res) => {
        handleRequest(account, req, res);
    })
        .on("listening", () => {
        account = createAccount(server.address());
    });
    return server;
}
exports.createHttpsServer = createHttpsServer;
exports.default = createHttpsServer;
