"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const path_match_1 = __importDefault(require("./path-match"));
const true_header_1 = __importDefault(require("./true-header"));
exports.default = (rules) => {
    const routes = {};
    Object.entries(rules).forEach(([method, route]) => {
        routes[method] = Object.entries(route).map(([path, handler]) => [
            (0, path_match_1.default)(path),
            handler
        ]);
    });
    return (req) => {
        let { pathname } = (0, url_1.parse)(req.url);
        // fix @azure/cosmos sends double slash url
        if (pathname && pathname.slice(0, 2) === "//") {
            pathname = pathname.slice(1);
        }
        let { method } = req;
        if (req.method === "POST" &&
            ((0, true_header_1.default)(req, "x-ms-documentdb-isquery") ||
                req.headers["content-type"] === "application/query+json")) {
            method += "_QUERY";
        }
        else if ((0, true_header_1.default)(req, "x-ms-documentdb-is-upsert")) {
            method += "_UPSERT";
        }
        else if ((0, true_header_1.default)(req, "x-ms-cosmos-is-batch-request")) {
            method += "_BATCH";
        }
        const methodRoutes = routes[method] || [];
        for (let i = 0, l = methodRoutes.length; i < l; i += 1) {
            const [match, handler] = methodRoutes[i];
            const params = match(pathname);
            if (params) {
                return [params, handler];
            }
        }
        return undefined;
    };
};
