"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parsePartitionKey = void 0;
const PARTITION_HEADER_KEY = `x-ms-documentdb-partitionkey`;
function transform(value, defaultValue) {
    if (!value) {
        return value;
    }
    return typeof value === "object" ? defaultValue : value;
}
/**
 * @returns partition head from incoming request if it exists
 */
function getPartitionFromHeader(req) {
    const header = req.headers[PARTITION_HEADER_KEY];
    return Array.isArray(header) ? header[0] : header;
}
exports.default = getPartitionFromHeader;
function parsePartitionKey(partitionKey, defaultValue) {
    let parsedHeader;
    try {
        parsedHeader = JSON.parse(partitionKey);
    }
    catch (error) {
        throw new Error(`Failed to parse ${PARTITION_HEADER_KEY} headers: ${partitionKey}`);
    }
    if (typeof parsedHeader === "string") {
        return transform(parsedHeader, defaultValue);
    }
    if (Array.isArray(parsedHeader)) {
        return transform(parsedHeader[0], defaultValue);
    }
    throw new Error(`Failed to parse ${PARTITION_HEADER_KEY} headers: ${partitionKey}`);
}
exports.parsePartitionKey = parsePartitionKey;
